@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.6/apps/mcping/mcping.cc 8499 $
-// @(#) $Id: mcping.cc 8499 2016-05-09 00:59:08Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.7/apps/mcping/mcping.cc 9488 $
+// @(#) $Id: mcping.cc 9488 2017-06-09 22:40:32Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2016
 //  All rights reserved.
@@ -45,13 +45,26 @@
 #include "DwmSignal.hh"
 #include "DwmSvnTag.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.6/apps/mcping/mcping.cc 8499 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.7/apps/mcping/mcping.cc 9488 $");
 
-static Dwm::OptArgs  g_optargs;
-static uint64_t      g_responses = 0;
+static Dwm::OptArgs      g_optargs;
+static uint64_t          g_responses = 0;
+static Dwm::HostPinger  *g_pinger = nullptr;
+static Dwm::Signal       g_sigInt(SIGINT);
 
 using namespace std;
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void HandleSigInt(int signum)
+{
+  cerr << "Stopping sender..." << flush;
+  g_pinger->StopSender();
+  cerr << " sender stopped\n";
+  g_sigInt.PopHandler();
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -81,26 +94,35 @@ class PingReceiver
   : public Dwm::HostPinger::Receiver
 {
 public:
+  PingReceiver(bool quiet)
+      : _quiet(quiet)
+  {}
+  
   void operator () (const Dwm::PingDestination & destination,
                     const Dwm::TimeValue & recvTime,
                     const Dwm::TimeValue & rtt)
   {
     if (rtt.Secs() != 0xFFFFFFFF) {
       ++g_responses;
-      if ((! destination.Hostname().empty()) && (! g_optargs.Get<bool>('n'))) {
-        cout << destination.Hostname();
+      if (! _quiet) {
+        if ((! destination.Hostname().empty()) && (! g_optargs.Get<bool>('n'))) {
+          cout << destination.Hostname();
+        }
+        else {
+          cout << destination.Address();
+        }
+        if (destination.Port()) {
+          cout << ":" << destination.Port();
+        }
+        cout << " " << setw(6)
+             << (double)(rtt.Secs() * 1000.0) + (rtt.Usecs() / 1000.0)
+             << " ms\n";
+        cout.flush();
       }
-      else {
-        cout << destination.Address();
-      }
-      if (destination.Port()) {
-        cout << ":" << destination.Port();
-      }
-      cout << " " << (double)(rtt.Secs() * 1000.0) + (rtt.Usecs() / 1000.0)
-           << " ms\n";
-      cout.flush();
     }
   }
+private:
+  bool _quiet;
 };
 
 //----------------------------------------------------------------------------
@@ -115,6 +137,8 @@ int main(int argc, char *argv[])
                       "network interface on which to listen");
   g_optargs.AddOptArg("n", "nonames", false, "0", "don't show host names");
   g_optargs.AddOptArg("p:", "pps", false, "1", "packet rate (packets/sec)");
+  g_optargs.AddOptArg("q", "quiet", false, "false", "quiet");
+  g_optargs.AddOptArg("t:", "timeout", false, "5", "timeout (seconds)");
   g_optargs.AddNormalArg("host(s)", true);
   
   int  nextArg = g_optargs.Parse(argc, argv);
@@ -128,8 +152,9 @@ int main(int argc, char *argv[])
     interface = GetLikelyInterface();
   }
 
-  Dwm::Signal  sigInt(SIGINT);
-  sigInt.Block();
+  Dwm::PingSequencesInFlight::TimeoutSecs(g_optargs.Get<uint32_t>('t'));
+  
+  g_sigInt.PushHandler(HandleSigInt);
 
   vector<Dwm::PingDestination>  destinations;
   Dwm::HostPinger  pinger(getpid() & 0xFFFF, interface);
@@ -139,23 +164,50 @@ int main(int argc, char *argv[])
     pinger.AddDestination(dst);
   }
   pinger.PacketRate(g_optargs.Get<int>('p'));
-  pinger.SetReceiver(new PingReceiver());
+  pinger.SetReceiver(new PingReceiver(g_optargs.Get<bool>('q')));
+  g_pinger = &pinger;
   pinger.Start(g_optargs.Get<int>('c'));
-
+  
   if (g_optargs.Get<int>('c')) {
-    while (pinger.Outstanding()) {
-      sleep(1);
+    while (! pinger.SenderDone()) {
+      usleep(250000);
+    }
+    if (pinger.Outstanding()) {
+      cerr << "Waiting up to "
+           << Dwm::PingSequencesInFlight::TimeoutSecs()
+           << " second"
+           << ((Dwm::PingSequencesInFlight::TimeoutSecs() > 1) ? "s" : "")
+           << " for outstanding packets...\n";
+      while (pinger.Outstanding()) {
+        usleep(250000);
+      }
     }
     pinger.Stop();
   }
   else {
-    sigInt.Wait();
+    g_sigInt.Wait();
+    pinger.StopSender();
+    if (pinger.Outstanding()) {
+      cerr << "Waiting up to "
+           << Dwm::PingSequencesInFlight::TimeoutSecs()
+           << " second"
+           << ((Dwm::PingSequencesInFlight::TimeoutSecs() > 1) ? "s" : "")
+           << " for outstanding packets...\n";
+      while (pinger.Outstanding()) {
+        usleep(250000);
+      }
+    }
     pinger.Stop();
   }
 
   for (int i = 0; i < destinations.size(); ++i) {
     cout << endl
-         << "--- " << destinations[i].Address() << " ping statistics ---"
+         << "--- " << destinations[i];
+    string  hn = destinations[i].Hostname();
+    if (! hn.empty()) {
+      cout << " (" << hn << ')';
+    }
+    cout << " ping statistics ---"
          << endl
          << pinger.PacketsSent(destinations[i]) 
          << " packets sent, " 
