@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: DwmHostPinger.cc 8499 2016-05-09 00:59:08Z dwm $
+// @(#) $Id: DwmHostPinger.cc 9487 2017-06-09 22:31:58Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2007, 2016
 //  All rights reserved.
@@ -46,6 +46,7 @@ extern "C" {
   #include <netinet/ip.h>
   #include <errno.h>
   #include <netdb.h>
+  #include <arpa/inet.h>
 }
 
 #include <cstring>
@@ -56,11 +57,13 @@ extern "C" {
 #include "DwmIpv4IcmpEchoReply.hh"
 #include "DwmIpv4IcmpEchoRequest.hh"
 #include "DwmIpv4TcpHeader.hh"
+#include "DwmSignal.hh"
 #include "DwmStringUtils.hh"
 #include "DwmHostPinger.hh"
 #include "DwmPacer.hh"
+#include "DwmPthreadSignal.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.6/src/DwmHostPinger.cc 8499 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.7/src/DwmHostPinger.cc 9487 $");
 
 using namespace std;
 
@@ -73,7 +76,7 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   HostPinger::HostPinger(uint16_t identifier, const std::string & device)
       : _myAddress(), _device(device), _watcherThread(), _senderThread(),
-        _watcherReadyMutex(),
+        _senderDone(false), _watcherReadyMutex(),
         _watcherReadyLock(_watcherReadyMutex, std::defer_lock),
         _watcherReady(), _runSender(false), _runWatcher(false),
         _identifier(identifier), _socket(), _packetRate(1),
@@ -82,10 +85,27 @@ namespace Dwm {
     char  myhostname[256];
     memset(myhostname, 0, 256);
     gethostname(myhostname, 255);
-    struct hostent  *hostEntry = gethostbyname(myhostname);
-    if (hostEntry) {
-      _myAddress = *(ipv4addr_t *)hostEntry->h_addr_list[0];
+    struct addrinfo  hints, *addrResult = 0;
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_INET;
+    hints.ai_socktype = SOCK_STREAM;
+    if (getaddrinfo(myhostname, nullptr, &hints, &addrResult) == 0) {
+      const struct addrinfo  *addr;
+      for (addr = addrResult; addr; addr = addr->ai_next) {
+        const struct sockaddr_in  *sain =
+          (const struct sockaddr_in *)addr->ai_addr;
+        if ((sain->sin_addr.s_addr & inet_addr("127.0.0.0")) !=
+            inet_addr("127.0.0.0")) {
+          _myAddress = sain->sin_addr.s_addr;
+          struct in_addr  inAddr;
+          inAddr.s_addr = _myAddress.Raw();
+          break;
+        }
+      }
     }
+    freeaddrinfo(addrResult);
+    struct in_addr  inAddr;
+    inAddr.s_addr = _myAddress.Raw();
   }
 
   //--------------------------------------------------------------------------
@@ -181,18 +201,19 @@ namespace Dwm {
   bool HostPinger::Start(int count)
   {
     bool  rc = false;
+    _packetsDesired = count;
     _count = count;
     if (_count > 0) {
-      // ++_count;
       _forever = false;
     }
-    if (! _pcap.OpenLive(_device, 128, false, 1000)) {
+    if (! _pcap.OpenLive(_device, 128, false, 500)) {
       cerr << "_pcap.OpenLive() failed: " << _pcap.LastError() << endl;
       return(false);
+      // _pcap.SetBufferSize(4096*1024);
     }
     
     ostringstream  filter;
-    filter << "(tcp and (src port 1 or dst port 1))"
+    filter << "(tcp port 1)"
            << " or (icmp[4:2] == " << _identifier << ")";
     _pcap.SetFilter(filter.str(), true);
     _runWatcher = true;
@@ -221,26 +242,31 @@ namespace Dwm {
   bool HostPinger::Stop()
   {
     bool  rc = false;
-    if (StopSender()) {
-      if (Outstanding()) {
-        sleep(1);
-      }
-      if (StopWatcher()) {
+#if 0
+    if (StopWatcher()) {
+      if (StopSender()) {
         rc = true;
       }
     }
+#else
+    if (! StopWatcher()) {
+      cerr << "Failed to stop watcher\n";
+    }
+    if (! StopSender()) {
+      cerr << "Failed to stop sender\n";
+    }
+#endif
     return(rc);
   }
   
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
-  bool HostPinger::Outstanding() const
+  bool HostPinger::Outstanding() // const
   {
     bool  rc = false;
-    map<PingDestination,PingDestinationEntry>::const_iterator  i;
-    for (i = _destinations.begin(); i != _destinations.end(); ++i) {
-      if (i->second.Sequences().Outstanding()) {
+    for (auto i : _destinations) {
+      if (i.second.Sequences().Outstanding()) {
         rc = true;
         break;
       }
@@ -256,10 +282,20 @@ namespace Dwm {
     bool  rc = false;
     _runWatcher = false;
     if (_watcherThread.joinable()) {
+      _pcap.BreakLoop();
+#ifdef __linux__
+      //  pcap_breakloop() nor timeouts work on linux (which makes it a crappy
+      //  operating system for pcap programs).
+      pthread_cancel(_watcherThread.native_handle());
+#endif
       _watcherThread.join();
       rc = true;
       _pcap.Close();
     }
+    else {
+      //  Watcher not running
+      rc = true;
+    }
     return(rc);
   }
   
@@ -277,6 +313,10 @@ namespace Dwm {
         _socket.Close();
       }
     }
+    else {
+      //  Sender not running
+      rc = true;
+    }
     return(rc);
   }
   
@@ -288,14 +328,18 @@ namespace Dwm {
                                 const uint8_t *pkt)
   {
     HostPinger  *pinger = (HostPinger *)user;
-    if (pchdr->caplen < (sizeof(struct ip) + pinger->_pcap.DataLinkOffset()))
+    if (pchdr->caplen < (sizeof(struct ip) + pinger->_pcap.DataLinkOffset())) {
+      cerr << "IP packet too short\n";
       return;
+    }
     struct ip  *ipptr = (struct ip *)(pkt + pinger->_pcap.DataLinkOffset());
     Ipv4PacketHeader  ip(ipptr);
     if (ip.Protocol() == IPPROTO_TCP) {
       if (pchdr->caplen <
-          ip.HeaderLength() + 20 + pinger->_pcap.DataLinkOffset())
+          ip.HeaderLength() + 20 + pinger->_pcap.DataLinkOffset()) {
+        cerr << "TCP packet too short\n";
         return;
+      }
       uint8_t  *ptr = (uint8_t *)pkt + pinger->_pcap.DataLinkOffset();
       ptr += ip.HeaderLength();
       Ipv4TcpHeader  tcpHeader((struct tcphdr *)ptr);
@@ -307,8 +351,10 @@ namespace Dwm {
           return;
         //  if the sequence number is 0, it's a TCP SYN we sent to prime
         //  pf and ipfw rules.  Ignore it.
+#if 0
         if (tcpHeader.SequenceNumber() == 0)
           return;
+#endif
         if (ip.Id() == pinger->_identifier) {
           iter->second.Sequences().AddSent(tcpHeader.SequenceNumber(),
                                            pchdr->ts);
@@ -318,8 +364,13 @@ namespace Dwm {
       else if (tcpHeader.DestinationPort() == 1) {
         PingDestination  destination(ip.SourceAddress(), tcpHeader.SourcePort());
         auto  iter = pinger->_destinations.find(destination);
-        if (iter == pinger->_destinations.end())  
+        if (iter == pinger->_destinations.end()) {
           return;
+        }
+        if (tcpHeader.Flags() & (TH_SYN|TH_ACK)) {
+          tcpHeader.SequenceNumber(0);
+        }
+        
         TimeValue  rtt;
         if (iter->second.Sequences().ProcessReceive(tcpHeader.SequenceNumber(),
                                                     pchdr->ts, rtt)) {
@@ -372,9 +423,20 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   void HostPinger::RunWatcher()
   {
+    Pthread::Signal  sigInt(SIGINT);
+    sigInt.BlockInThread();
+#ifdef __linux__
+    //  pcap_breakloop() nor timeouts work on linux (which makes it a crappy
+    //  operating system for pcap programs).  So set our cancellation type
+    //  to asynchronous.
+    int  oldType;
+    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &oldType);
+#endif    
     _watcherReady.notify_all();
     while (_runWatcher) {
-      _pcap.Dispatch(-1, HostPinger::PcapCallback, (uint8_t *)this);
+      if (_pcap.Dispatch(-1, HostPinger::PcapCallback, (uint8_t *)this) < 0) {
+        break;
+      }
     }
     return;
   }
@@ -405,6 +467,7 @@ void HostPinger::GetTcpPacket(const PingDestination & destination,
     tcpHeader.DataOffset(20);
     if (! _destinations[destination].Primed()) {
       tcpHeader.Flags(TH_SYN);
+      // tcpHeader.Flags(TH_RST);
       _destinations[destination].Primed(true);
     }
     else {
@@ -425,10 +488,14 @@ void HostPinger::GetTcpPacket(const PingDestination & destination,
   //--------------------------------------------------------------------------
   void HostPinger::RunSender()
   {
+    _senderDone = false;
+    Pthread::Signal  sigInt(SIGINT);
+    sigInt.BlockInThread();
+    
     Pacer  pacer(_packetRate);
     uint16_t  seqno = 0;
     
-    while (_runSender || _count > 0) {
+    while (_runSender || (_runSender && (_count > 0))) {
       string  packet;
       map<PingDestination,PingDestinationEntry>::iterator  i;
       for (i = _destinations.begin(); 
@@ -441,18 +508,45 @@ void HostPinger::GetTcpPacket(const PingDestination & destination,
       if (! _forever) {
         --_count;
         if (_count <= 0) {
-          if (Outstanding()) {
-            sleep(2);
+          int  numWaits = 5;
+          while ((! AllPacketsSent()) || Outstanding()) {
+            usleep(250000);
+            --numWaits;
+            if (numWaits <= 0) {
+              break;
+            }
           }
-          StopWatcher();
+          _senderDone = true;
           return;
         }
       }
       ++seqno;
     }
+    _senderDone = true;
     return;
   }
-  
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool HostPinger::SenderDone() const
+  {
+    return _senderDone;
+  }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool HostPinger::AllPacketsSent() const
+  {
+    bool  rc = true;
+    for (auto i = _destinations.begin(); i != _destinations.end(); ++i) {
+      if (_packetsDesired > i->second.PacketsSent()) {
+        rc = false;
+        break;
+      }
+    }
+    return rc;
+  }
   
 }  // namespace Dwm
