@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: DwmPingDestinationEntry.cc 8499 2016-05-09 00:59:08Z dwm $
+// @(#) $Id: DwmPingDestinationEntry.cc 9482 2017-06-09 05:14:31Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2008
 //  All rights reserved.
@@ -45,12 +45,17 @@
 #include "DwmPingDestinationEntry.hh"
 #include "DwmSvnTag.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.6/src/DwmPingDestinationEntry.cc 8499 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.7/src/DwmPingDestinationEntry.cc 9482 $");
 
 using namespace std;
 
 namespace Dwm {
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  uint32_t  PingSequencesInFlight::_timeoutSecs = 5;
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -103,8 +108,9 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
-  bool PingSequencesInFlight::Outstanding() const
+  bool PingSequencesInFlight::Outstanding() // const
   {
+    CleanupOld();
     return(! _sequences.empty());
   }
   
@@ -113,7 +119,7 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   void PingSequencesInFlight::CleanupOld()
   {
-    TimeValue  old(time((time_t *)0) - 10, 0);
+    TimeValue  old(time((time_t *)0) - _timeoutSecs, 0);
     
     map<uint16_t,TimeValue>::iterator  i = _sequences.begin();
     while (i != _sequences.end()) {
@@ -131,6 +137,23 @@ namespace Dwm {
     return;
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  uint32_t PingSequencesInFlight::TimeoutSecs()
+  {
+    return _timeoutSecs;
+  }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  uint32_t PingSequencesInFlight::TimeoutSecs(uint32_t secs)
+  {
+    _timeoutSecs = secs;
+    return _timeoutSecs;
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
