@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: DwmSignal.cc 8388 2016-04-17 03:23:45Z dwm $
+// @(#) $Id: DwmSignal.cc 9485 2017-06-09 05:55:12Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2000-2007
 //  All rights reserved.
@@ -42,99 +42,82 @@
 #include "DwmSysLogger.hh"
 #include "DwmSvnTag.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.6/src/DwmSignal.cc 8388 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.7/src/DwmSignal.cc 9485 $");
 
 namespace Dwm {
 
-  //--------------------------------------------------------------------------
-  //!  
-  //--------------------------------------------------------------------------
-  void Signal::FillNames()
-  {
-    static Pthread::Mutex  mtx;
-    mtx.Lock();
-    
-    if (Signal::_names.empty()) {
-      Signal::_names[0] = "Unknown";
-      Signal::_names[SIGHUP] = "SIGHUP";
-      Signal::_names[SIGINT] = "SIGINT";
-      Signal::_names[SIGQUIT] = "SIGQUIT";
-      Signal::_names[SIGILL] = "SIGILL";
-      #ifdef SIGTRAP
-        Signal::_names[SIGTRAP] = "SIGTRAP";
-      #endif
-      Signal::_names[SIGABRT] = "SIGABRT";
-      #ifdef SIGEMT
-        Signal::_names[SIGEMT] = "SIGEMT";
-      #endif
-      Signal::_names[SIGFPE] = "SIGFPE";
-      Signal::_names[SIGKILL] = "SIGKILL";
-      #ifdef SIGBUS
-        Signal::_names[SIGBUS] = "SIGBUS";
-      #endif
-      Signal::_names[SIGSEGV] = "SIGSEGV";
-      #ifdef SIGSYS
-        Signal::_names[SIGSYS] = "SIGSYS";
-      #endif
-      Signal::_names[SIGPIPE] = "SIGPIPE";
-      Signal::_names[SIGALRM] = "SIGALRM";
-      Signal::_names[SIGTERM] = "SIGTERM";
-      #ifdef SIGURG
-        Signal::_names[SIGURG] = "SIGURG";
-      #endif
-      Signal::_names[SIGSTOP] = "SIGSTOP";
-      Signal::_names[SIGTSTP] = "SIGTSTP";
-      Signal::_names[SIGCONT] = "SIGCONT";
-      Signal::_names[SIGCHLD] = "SIGCHLD";
-      Signal::_names[SIGTTIN] = "SIGTTIN";
-      Signal::_names[SIGTTOU] = "SIGTTOU";
-      #ifdef SIGIO
-        Signal::_names[SIGIO] = "SIGIO";
-      #endif
-      #ifdef SIGXCPU
-        Signal::_names[SIGXCPU] = "SIGXCPU";
-      #endif
-      #ifdef SIGXFSZ
-        Signal::_names[SIGXFSZ] = "SIGXFSZ";
-      #endif
-      #ifdef SIGVTALRM
-        Signal::_names[SIGVTALRM] = "SIGVTALRM";
-      #endif
-      #ifdef SIGPROF
-        Signal::_names[SIGPROF] = "SIGPROF";
-      #endif
-      #ifdef SIGWINCH
-        Signal::_names[SIGWINCH] = "SIGWINCH";
-      #endif
-      #ifdef SIGINFO
-        Signal::_names[SIGINFO] = "SIGINFO";
-      #endif
-        
-      Signal::_names[SIGUSR1] = "SIGUSR1";
-      Signal::_names[SIGUSR2] = "SIGUSR2";
-    }
-    mtx.Unlock();
-  }
+  const std::map<int,std::string>  Signal::_names = {
+    { 0, "Unknown" },
+    { SIGHUP, "SIGHUP" },
+    { SIGINT, "SIGINT" },
+    { SIGQUIT, "SIGQUIT" },
+    { SIGILL, "SIGILL" },
+#ifdef SIGTRAP
+    { SIGTRAP, "SIGTRAP" },
+#endif
+    { SIGABRT, "SIGABRT" },
+#ifdef SIGEMT
+    { SIGEMT, "SIGEMT" },
+#endif
+    { SIGFPE, "SIGFPE" },
+    { SIGKILL, "SIGKILL" },
+#ifdef SIGBUS
+    { SIGBUS, "SIGBUS" },
+#endif
+    { SIGSEGV, "SIGSEGV" },
+#ifdef SIGSYS
+    { SIGSYS, "SIGSYS" },
+#endif
+    { SIGPIPE, "SIGPIPE" },
+    { SIGALRM, "SIGALRM" },
+    { SIGTERM, "SIGTERM" },
+#ifdef SIGURG
+    { SIGURG, "SIGURG" },
+#endif
+    { SIGSTOP, "SIGSTOP" },
+    { SIGTSTP, "SIGTSTP" },
+    { SIGCONT, "SIGCONT" },
+    { SIGCHLD, "SIGCHLD" },
+    { SIGTTIN, "SIGTTIN" },
+    { SIGTTOU, "SIGTTOU" },
+#ifdef SIGIO
+    { SIGIO, "SIGIO" },
+#endif
+#ifdef SIGXCPU
+    { SIGXCPU, "SIGXCPU" },
+#endif
+#ifdef SIGXFSZ
+    { SIGXFSZ, "SIGXFSZ" },
+#endif
+#ifdef SIGVTALRM
+    { SIGVTALRM, "SIGVTALRM" },
+#endif
+#ifdef SIGPROF
+    { SIGPROF, "SIGPROF" },
+#endif
+#ifdef SIGWINCH
+    { SIGWINCH, "SIGWINCH" },
+#endif
+#ifdef SIGINFO
+    { SIGINFO, "SIGINFO" },
+#endif
+    { SIGUSR1, "SIGUSR1" },
+    { SIGUSR2, "SIGUSR2" }
+  };
   
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
   Signal::Signal()
   : _sigNum(-1)
-  {
-    if (Signal::_names.empty())
-      Signal::FillNames();
-  }
+  { }
   
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
   Signal::Signal(int sigNum)
     : _sigNum(sigNum)
-  {
-    if (Signal::_names.empty())
-      Signal::FillNames();
-  }
+  { }
   
   //--------------------------------------------------------------------------
   //!  
@@ -247,8 +230,8 @@ namespace Dwm {
     action.sa_flags   = 0;
 
     if (sigaction(this->_sigNum,&action,NULL) < 0) {
-      Syslog(LOG_ERR,"sigaction(%d,%p,NULL) failed {%s:%d}",
-             this->_sigNum,&action,__FILE__,__LINE__);
+      Syslog(LOG_ERR, "sigaction(%d,%p,NULL) failed",
+             this->_sigNum, &action);
     }
     else {
       this->_handlers.push_front(handler);
@@ -292,14 +275,11 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   const std::string & Signal::Name() const
   {
-    std::map<int,std::string>::const_iterator iter;
-    iter = Signal::_names.find(this->_sigNum);
+    auto  iter = Signal::_names.find(this->_sigNum);
     if (iter != Signal::_names.end())
       return(iter->second);
     else
-      return(Signal::_names[0]);
+      return(Signal::_names.find(0)->second);
   }
     
-  std::map<int,std::string>  Signal::_names;
-
 }  // namespace Dwm
